<?php

namespace Iwaydev\Rbac\Listeners;

use Iwaydev\Rbac\Components\RbacClient;
use Iwaydev\Rbac\Events\UpdatePermissions as UpdatePermissionsEvent;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\DB;
use Psr\Log\LoggerInterface;

/**
 * Обработчик события обновления прав доступа
 *
 * Class UpdatePermissions
 *
 * @package Rbac\Listeners
 */
class UpdatePermissions
{
    /** @var LoggerInterface|null Логгер */
    private ?LoggerInterface $logger;
    /** @var string */
    private string $xUuid = '';

    /**
     * Handle the event.
     *
     * @param UpdatePermissionsEvent $event
     *
     * @return void
     * @throws GuzzleException
     * @throws \Exception
     */
    public function handle(UpdatePermissionsEvent $event)
    {
        $this->logger = $event->getLogger();
        $rbacClient = $event->getRbacClient();
        $this->xUuid = $rbacClient->getXUuid();
        $this->logInfo('---Начало обновления прав доступа---');
        try {
            $roles = $rbacClient->getRoles([
                'project_name' => config('senders.rbac.connection.project_name'),
                'relations' => [
                    'permissions',
                    'users',
                ]
            ]);
        } catch (\Exception $exception) {
            $this->logError('Exception при получении ролей: ' . $exception->getMessage());
            $this->logException($exception);
        }
        $this->logInfo('Полученные роли', $roles);
        if (empty($roles)) {
            $this->logInfo('---Права доступа отсутствуют---');

            return;
        }
        if (!is_array($roles)) {
            $this->logError('Ошибка!!! Роли не являются массивом: ' . print_r($roles, true));

            return;
        }
        DB::beginTransaction();
        try {
            $this->update($roles);
        } catch (\Exception $exception) {
            DB::rollBack();
            $this->logError('Exception при обновлении: ' . $exception->getMessage());
            $this->logException($exception);

            return;
        }
        DB::commit();
        $this->logInfo('---Права доступа успешно обновлены---');
    }

    /**
     * Обновление ролей
     *
     * @param array $roles Роли
     */
    private function update(array $roles)
    {
        DB::table('rbac.roles_users')->truncate();
        DB::table('rbac.roles_permissions')->truncate();
        DB::table('rbac.roles')->truncate();
        DB::table('rbac.permissions')->truncate();
        foreach ($roles as $role) {
            DB::table('rbac.roles')->updateOrInsert(['name' => $role['name']]);
            if (!empty($role['permissions'])) {
                foreach ($role['permissions'] as $permission) {
                    DB::table('rbac.permissions')->updateOrInsert(['name' => $permission]);
                    DB::table('rbac.roles_permissions')->updateOrInsert([
                        'role_name' => $role['name'],
                        'permission_name' => $permission
                    ]);
                }
            }
            if (!empty($role['users'])) {
                foreach ($role['users'] as $userId) {
                    DB::table('rbac.roles_users')->updateOrInsert([
                        'role_name' => $role['name'],
                        'user_id' => $userId
                    ]);
                }
            }
        }
    }


    /**
     * Сделать запись уровня "info" в лог, если логгер установлен
     *
     * @param string $message Сообщение
     * @param array $context Дополнительные данные
     */
    private function logInfo(string $message, array $context = []): void
    {
        if (!empty($this->logger)) {
            $this->logger->info($this->xUuid . ' ' . $message, $context);
        }
    }

    /**
     * Сделать запись уровня "error" в лог, если логгер установлен
     *
     * @param string $message Сообщение
     * @param array $context Дополнительные данные
     */
    private function logError(string $message, array $context = []): void
    {
        if (!empty($this->logger)) {
            $this->logger->error($this->xUuid . ' ' . $message, $context);
        }
    }

    /**
     * Логируем исключение
     *
     * @param \Throwable $throwable Экзепляр исключения
     */
    public function logException(\Throwable $throwable): void
    {
        $this->logError('');
        $this->logError('Исключение!');
        $this->logError('Сообщение: ' . $throwable->getMessage());
        $this->logError('Код: ' . $throwable->getCode());
        $this->logError('Файл: ' . $throwable->getFile() . '(' . $throwable->getLine() . ')');
        $this->logError('Трейс: ' . PHP_EOL . $throwable->getTraceAsString());
        $this->logError('');
    }
}
